function capitalizeFirstLetter( string ) {
    return string.charAt( 0 ).toUpperCase() + string.slice( 1 );
}

function lowerFirstLetter( string ) {
    return string.charAt( 0 ).toLowerCase() + string.slice( 1 );
}

function prepareInsertQuery( attributes ) {
	let columnCount = 0;
	let str = 'INSERT INTO %s (';
	for ( attribute of attributes ) {
		columnCount++;
		str += `${attribute.tableAttribute}`;
		if ( columnCount < attributes.length ) {
			str += ', ';
		}
	}
	str += ') VALUES (';
	columnCount = 0;
	for ( attribute of attributes ) {
		columnCount++;
		str += `:${attribute.tableAttribute}`;
		if ( columnCount < attributes.length ) {
			str += ', ';
		}
	}
	str += ') ON DUPLICATE KEY UPDATE ';
	columnCount = 0;
	for ( attribute of attributes ) {
		columnCount++;
		if ( 'id' === attribute.tableAttribute ) {
			continue;
		}
		str += `${attribute.tableAttribute} = VALUES(${attribute.tableAttribute})`;
		if ( columnCount < attributes.length ) {
			str += ', ';
		}
	}
	return str;
}

function addParamValue( attributes, pojoName ) {
	let str = '';
	let i = 0;
	let attrCount = Object.keys( attributes ).length;
	for ( attribute of attributes ) {
		i++
		str += `.addValue( "${attribute.tableAttribute}", ${lowerFirstLetter(mapping.pojoName)}.get${capitalizeFirstLetter(attribute.pojoAttribute)}() )`;
		if ( i < attrCount ) {
			str += `\n\t\t\t`;
		}
	
	}
	return str;
}

exports.createFileContent = function( mapping, payload ) {
	return `import ${payload.modelPackage}.${mapping.pojoName};
import ${payload.basePackage}.util.KeyHolderFunctions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;

@Repository
public class ${mapping.pojoName}DAO {
	@Autowired
	private ${payload.dataSourceType} ${payload.dataSourceName};

	private final String TABLE_NAME = "${payload.databaseName}.${mapping.tableName}";
	private final String SELECT_QUERY = String.format( "SELECT * FROM %s WHERE id = :id", TABLE_NAME );
	private final String SELECT_ALL_QUERY = String.format( "SELECT * FROM %s ORDER BY id DESC", TABLE_NAME );
	private final String DELETE_QUERY = String.format( "DELETE FROM %s WHERE id = :id", TABLE_NAME );
	private final String INSERT_QUERY = String.format( "${prepareInsertQuery(mapping.attributes)}", TABLE_NAME );

	// prepare query params
	private MapSqlParameterSource prepareInsertQueryParams( ${mapping.pojoName} ${lowerFirstLetter(mapping.pojoName)} ) {
		MapSqlParameterSource params = new MapSqlParameterSource();
		params${addParamValue( mapping.attributes, mapping.pojoName )};
		return params;
	}

	// find
	public ${mapping.pojoName} find( long id ) {
		MapSqlParameterSource params = new MapSqlParameterSource();
		params.addValue( "id", id );
		List<${mapping.pojoName}> ${lowerFirstLetter(mapping.pojoName)}s = ${payload.dataSourceName}.query( SELECT_QUERY, params, new ${mapping.pojoName}RowMapper() );
		if ( ${lowerFirstLetter(mapping.pojoName)}s.size() > 0 ) {
			return ${lowerFirstLetter(mapping.pojoName)}s.get( 0 );
		} else {
			return new ${mapping.pojoName}();
		}
	}

	// find all
	public List<${mapping.pojoName}> findAll() {
		return ${payload.dataSourceName}.query( SELECT_ALL_QUERY, new ${mapping.pojoName}RowMapper() );
	}

	// save
	public long save( ${mapping.pojoName} ${lowerFirstLetter(mapping.pojoName)} ) {
		KeyHolder keyHolder = new GeneratedKeyHolder();
		MapSqlParameterSource params = prepareInsertQueryParams( ${lowerFirstLetter(mapping.pojoName)} );
		${payload.dataSourceName}.update( INSERT_QUERY, params, keyHolder );
		return KeyHolderFunctions.getKey( ${lowerFirstLetter(mapping.pojoName)}.getId(), keyHolder );
	}

	// save all
	public int[] saveAll( List<${mapping.pojoName}> ${lowerFirstLetter(mapping.pojoName)}s ) {
		List<MapSqlParameterSource> batchArgs = new ArrayList<>();
		for ( ${mapping.pojoName} ${lowerFirstLetter(mapping.pojoName)}: ${lowerFirstLetter(mapping.pojoName)}s ) {
			MapSqlParameterSource params = prepareInsertQueryParams( ${lowerFirstLetter(mapping.pojoName)} );
			batchArgs.add( params );
		}
		return ${payload.dataSourceName}.batchUpdate( INSERT_QUERY, batchArgs.toArray(new MapSqlParameterSource[${lowerFirstLetter(mapping.pojoName)}s.size()]) );
	}

	// delete
	public boolean delete( long id ) {
		MapSqlParameterSource params = new MapSqlParameterSource();
		params.addValue( "id", id );
		return ${payload.dataSourceName}.update( DELETE_QUERY, params ) > 0;
	}
}`;
};